(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    208304,       4366]
NotebookOptionsPosition[    204851,       4243]
NotebookOutlinePosition[    205195,       4258]
CellTagsIndexPosition[    205152,       4255]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Reale Beispiele f\[UDoubleDot]r die Anwendung des HUPP packages", \
"Section"],

Cell["\<\
Dieses Notebook hier, soll die reale Anwedung des HUPP - Packages \
demonstrieren.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Laden des Packages", "Section"],

Cell["Entweder \[UDoubleDot]ber Needs (ignoriert das ClearAll)", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "\"\<HUPP`*\>\"", "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"Needs", "[", "\"\<HUPP`\>\"", "]"}]}], "Input"],

Cell["Oder \[UDoubleDot]ber den optionalen Befehl", "Text"],

Cell[BoxData[
 RowBox[{"HUPP", "[", "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Beispiel: F3 - Fadenpendel: Zeitmessung", "Section"],

Cell[CellGroupData[{

Cell["Datenaufnahme", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dataF3", "=", 
   RowBox[{"{", 
    RowBox[{
    "20.63", ",", "20.66", ",", "20.68", ",", "20.72", ",", "20.72", ",", 
     "20.66", ",", "20.72", ",", "20.78", ",", "20.68", ",", "20.59"}], 
    "}"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Berechnung der Zeitmessung mit Reaktionsfehler: 0.06 - systematischer Ger\
\[ADoubleDot]tefehler von 0.01+5*10^(-4)*t und zuf\[ADoubleDot]lligen Fehler\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalculateMeasurement", "[", 
  RowBox[{"dataF3", ",", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"0.06", ",", "\[IndentingNewLine]", 
     RowBox[{"0.01", "+", 
      RowBox[{"5", "*", 
       RowBox[{"10", "^", 
        RowBox[{"(", 
         RowBox[{"-", "4"}], ")"}]}], "*", "C"}]}]}], "\[IndentingNewLine]", 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"20.684`", ",", "0.06586434492839308`"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gleiche Berechnung mit Analyse", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalculateMeasurement", "[", 
  RowBox[{"dataF3", ",", 
   RowBox[{"{", 
    RowBox[{"0.06", ",", 
     RowBox[{"0.01", "+", 
      RowBox[{"5", "*", 
       RowBox[{"10", "^", 
        RowBox[{"(", 
         RowBox[{"-", "4"}], ")"}]}], "*", "C"}]}]}], "}"}], ",", 
   RowBox[{"Analyze", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {"\<\"\\!\\(\\*OverscriptBox[\\(x\\), \\(_\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(u\\), \\(sys\\)]\\)\"\>", \
"\<\"\\!\\(\\*OverscriptBox[\\(x\\), \
\\(_\\)]\\)+\\!\\(\\*SubscriptBox[\\(u\\), \\(sys\\)]\\)\"\>", "\<\"u\"\>", "\
\<\"\\!\\(\\*SubscriptBox[\\(u\\), \
\\(sys\\)]\\)/\\!\\(\\*OverscriptBox[\\(x\\), \\(_\\)]\\) [%]\"\>", \
"\<\"n\"\>", "\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(+\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(-\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(0\\)]\\)\"\>", \
"\<\"|\\!\\(\\*SubscriptBox[\\(n\\), \
\\(+\\)]\\)-\\!\\(\\*SubscriptBox[\\(n\\), \
\\(-\\)]\\)|\[LessEqual]\\!\\(\\*SqrtBox[\\(n\\)]\\)\"\>", "\<\"t\"\>", "\<\"\
\[Sigma]\"\>", "\<\"\\!\\(\\*SubscriptBox[\\(min\\), \
\\(i\\)]\\)|\\!\\(\\*OverscriptBox[\\(x\\), \
\\(_\\)]\\)-\\!\\(\\*SubscriptBox[\\(x\\), \\(i\\)]\\)|\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(max\\), \\(i\\)]\\)|\\!\\(\\*OverscriptBox[\\(x\
\\), \\(_\\)]\\)-\\!\\(\\*SubscriptBox[\\(x\\), \\(i\\)]\\)|\"\>"},
        {"20.684`", "0", "20.684`", "0.06586434492839308`", "0.`", "10", "6", 
         "4", "0", 
         ItemBox["\<\"2\[LessEqual]3.16228\"\>",
          Background->RGBColor[0.88, 1, 0.88],
          StripOnInput->False], "1.058727665741402`", "0.053789714010518255`",
          "0.004000000000001336`", "0.09600000000000009`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"uncertanity u\"\>", "\<\"\\!\\(\\*OverscriptBox[\\(s\\), \
\\(_\\)]\\)=t\[CenterDot]s/\\!\\(\\*SqrtBox[\\(n\\)]\\)\"\>", "0.06`", 
         RowBox[{"0.01`", "\[VeryThinSpace]", "+", 
          FractionBox["C", "2000"]}]},
        {"\<\"N[u]\"\>", "0.01800874700933797`", "0.06`", "0.020342`"},
        {"\<\"u^2\"\>", "0.0003243149688463393`", "0.0036`", 
         "0.00041379696399999996`"},
        {"\<\"uncertanty weight [%]\"\>", "7.475947459786924`", 
         "82.98541060552935`", "9.538641934683735`"},
        {"\<\"Sqrt-uncertanty weights [%]\"\>", "0.40204510268206883`", 
         "74.38048958003381`", "25.217465317284127`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     GraphicsBox[{{
        {RGBColor[0.92, 0.488, 0.2432], EdgeForm[{Opacity[0.6649999999999999],
          Thickness[Small]}], {}, 
         {RGBColor[0.92, 0.488, 0.2432], EdgeForm[{Opacity[
          0.6649999999999999], Thickness[Small]}], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{20.5, 0}, {20.6, 1}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{126.2467587090141, 249.98221593237758`}, {
               82.43477598385076, 112.41666666666663`}}],
             StatusArea[#, 1]& ,
             TagBoxNote->"1"],
            StyleBox["1", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[1, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{20.6, 0}, {20.7, 5}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{249.48221593237758`, 
               373.21767315573743`}, {-35.49278674741274, 
               112.41666666666663`}}],
             StatusArea[#, 5]& ,
             TagBoxNote->"5"],
            StyleBox["5", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[5, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{20.7, 0}, {20.8, 4}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{372.71767315573743`, 
               496.4531303791009}, {-6.010896064596864, 112.41666666666663`}}],
             StatusArea[#, 4]& ,
             TagBoxNote->"4"],
            StyleBox["4", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[4, {FontFamily -> "Times"}], 
            "Tooltip"]& ]}, {}, {}}, {{}, {}, {}}}, {{}, {{}, {}, 
         {RGBColor[1, 0, 0], PointSize[0.008333333333333333], 
          AbsoluteThickness[1.6], CapForm["Butt"], LineBox[CompressedData["
1:eJxF23VQlN/XAHC6u9lFUVGUEIklFPFeFZSU8gtKmIQ0oiKgIi1IioqS0ikK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           "]]}}, {}, {}, {{}, {}}}, {{{}, {}, 
         {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], CapForm[
          "Butt"], LineBox[CompressedData["
1:eJxF2nc8Vu/7AHAzkiQz47EyikL1mMl1KiGVHSVlZFN2ZGSvqMwIRVbKKqOl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           "]]}}, {}, {}}},
      AspectRatio->NCache[
        Rational[1, 3], 0.3333333333333333],
      Axes->{False, False},
      AxesLabel->{None, None},
      AxesOrigin->{20.404293487145306`, 0},
      Epilog->{
        GrayLevel[0.5], 
        Thickness[Large], 
        TagBox[
         ConicHullRegionBox[{{20.684, 0}, {20.684, 1}}], "InfiniteLine"], 
        Thickness[Tiny], 
        TagBox[
         ConicHullRegionBox[{{20.630210285989484`, 0}, {
           20.630210285989484`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.737789714010518`, 0}, {
           20.737789714010518`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.576420571978964`, 0}, {
           20.576420571978964`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.791579428021038`, 0}, {
           20.791579428021038`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.522630857968448`, 0}, {
           20.522630857968448`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.845369142031554`, 0}, {
           20.845369142031554`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.468841143957928`, 0}, {
           20.468841143957928`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.899158856042074`, 0}, {
           20.899158856042074`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.41505142994741, 0}, {20.41505142994741, 1}}],
          "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{20.95294857005259, 0}, {20.95294857005259, 1}}],
          "InfiniteLine"]},
      Frame->True,
      FrameLabel->{{None, None}, {None, None}},
      FrameStyle->Automatic,
      FrameTicks->{{Automatic, Automatic}, {Automatic, {{20.684, 
           FormBox["\"\[Mu]\"", TraditionalForm]}, {20.737789714010518`, 
           FormBox["\"1\[Sigma]\"", TraditionalForm]}, {20.630210285989484`, 
           FormBox["\"-1\[Sigma]\"", TraditionalForm]}, {20.791579428021038`, 
           FormBox["\"2\[Sigma]\"", TraditionalForm]}, {20.576420571978964`, 
           FormBox["\"-2\[Sigma]\"", TraditionalForm]}, {20.845369142031554`, 
           FormBox["\"3\[Sigma]\"", TraditionalForm]}, {20.522630857968448`, 
           FormBox["\"-3\[Sigma]\"", TraditionalForm]}, {20.899158856042074`, 
           FormBox["\"4\[Sigma]\"", TraditionalForm]}, {20.468841143957928`, 
           FormBox["\"-4\[Sigma]\"", TraditionalForm]}, {20.95294857005259, 
           FormBox["\"5\[Sigma]\"", TraditionalForm]}, {20.41505142994741, 
           FormBox["\"-5\[Sigma]\"", TraditionalForm]}}}},
      GridLines->{{0}, {0}},
      GridLinesStyle->Directive[
        GrayLevel[0.5, 0.4]],
      ImageSize->700,
      LabelStyle->{FontFamily -> "Times"},
      PlotRange->{{20.41505142994741, 20.95294857005259}, {All, All}},
      PlotRangePadding->{{
         Scaled[0.02], 
         Scaled[0.02]}, {None, 
         Scaled[0.05]}},
      Ticks->{Automatic, Automatic}]}
   },
   AutoDelete->False,
   GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
       GrayLevel[1], 
       GrayLevel[0.85]}},
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"20.684`", ",", "0.06586434492839308`"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gleiche Berechnung in gerundeter Darstellung", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RoundedErrorForm", "[", 
  RowBox[{"CalculateMeasurement", "[", 
   RowBox[{"dataF3", ",", 
    RowBox[{"{", 
     RowBox[{"0.06", ",", 
      RowBox[{"0.01", "+", 
       RowBox[{"5", "*", 
        RowBox[{"10", "^", 
         RowBox[{"(", 
          RowBox[{"-", "4"}], ")"}]}], "*", "C"}]}]}], "}"}]}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["\<\"(2.068\[PlusMinus]0.007)\\!\\(\\*FormBox[TemplateBox[List[\\\
\"10\\\", \\\"1\\\"], \\\"Superscript\\\", Rule[SyntaxForm, SuperscriptBox]], \
TraditionalForm]\\)\"\>"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Beispiel: T7 - spezifische W\[ADoubleDot]rmekapaz. idealer gase: Gau\[SZ]sche \
Fehlerfortpflanzung d. Adiabatenexp.\
\>", "Section"],

Cell["Ohne analyse", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t7result", "=", 
  RowBox[{"GaussianErrorPropagation", "[", 
   RowBox[{
    RowBox[{"4", "*", "V", "*", 
     RowBox[{"m", "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"r", "^", "4"}], "*", "P", "*", 
        RowBox[{"t", "^", "2"}]}], ")"}]}]}], ",", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "0.004359", ",", "0.000011"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"m", ",", "0.00627", ",", "0.00001"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"r", ",", "0.006975", ",", "0.000005"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"P", ",", "104200", ",", "100"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"t", ",", "0.57465", ",", "0.000387325"}], "}"}]}], 
     "\[IndentingNewLine]", "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.3423414853318556`", ",", "0.005983881469889043`"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["Mit analyse", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GaussianErrorPropagation", "[", 
  RowBox[{
   RowBox[{"4", "*", "V", "*", 
    RowBox[{"m", "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"r", "^", "4"}], "*", "P", "*", 
       RowBox[{"t", "^", "2"}]}], ")"}]}]}], ",", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"V", ",", "0.004359", ",", "0.000011"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"m", ",", "0.00627", ",", "0.00001"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0.006975", ",", "0.000005"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"P", ",", "104200", ",", "100"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0.57465", ",", "0.000387325"}], "}"}]}], 
    "\[IndentingNewLine]", "}"}], ",", 
   RowBox[{"Analyze", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {"\<\"Parameters\"\>", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"V", ",", "0.004359`", ",", "0.000011`"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"m", ",", "0.00627`", ",", "0.00001`"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"r", ",", "0.006975`", ",", "5.`*^-6"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"P", ",", "104200", ",", "100"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"t", ",", "0.57465`", ",", "0.000387325`"}], "}"}]}], 
          "}"}]}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {
           GrayLevel[0.85], 
           GrayLevel[1]}, "Rows" -> {{None}}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"Function\"\>", "\<\"Uncertanty\"\>"},
        {
         FractionBox[
          RowBox[{"4", " ", "m", " ", "V"}], 
          RowBox[{"P", " ", 
           SuperscriptBox["r", "4"], " ", 
           SuperscriptBox["t", "2"]}]], 
         SqrtBox[
          RowBox[{
           FractionBox[
            RowBox[{"16", " ", 
             SuperscriptBox["V", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "m", "]"}], "2"]}], 
            RowBox[{
             SuperscriptBox["P", "2"], " ", 
             SuperscriptBox["r", "8"], " ", 
             SuperscriptBox["t", "4"]}]], "+", 
           FractionBox[
            RowBox[{"16", " ", 
             SuperscriptBox["m", "2"], " ", 
             SuperscriptBox["V", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "P", "]"}], "2"]}], 
            RowBox[{
             SuperscriptBox["P", "4"], " ", 
             SuperscriptBox["r", "8"], " ", 
             SuperscriptBox["t", "4"]}]], "+", 
           FractionBox[
            RowBox[{"256", " ", 
             SuperscriptBox["m", "2"], " ", 
             SuperscriptBox["V", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "r", "]"}], "2"]}], 
            RowBox[{
             SuperscriptBox["P", "2"], " ", 
             SuperscriptBox["r", "10"], " ", 
             SuperscriptBox["t", "4"]}]], "+", 
           FractionBox[
            RowBox[{"64", " ", 
             SuperscriptBox["m", "2"], " ", 
             SuperscriptBox["V", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "t", "]"}], "2"]}], 
            RowBox[{
             SuperscriptBox["P", "2"], " ", 
             SuperscriptBox["r", "8"], " ", 
             SuperscriptBox["t", "6"]}]], "+", 
           FractionBox[
            RowBox[{"16", " ", 
             SuperscriptBox["m", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "V", "]"}], "2"]}], 
            RowBox[{
             SuperscriptBox["P", "2"], " ", 
             SuperscriptBox["r", "8"], " ", 
             SuperscriptBox["t", "4"]}]]}]]},
        {"1.3423414853318556`", "0.005983881469889043`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"Type/Variable\"\>", "\<\"Formula\"\>", "\<\"Value\"\>", \
"\<\"Weights [%]\"\>", "\<\"Relative Weights [%]\"\>", "\<\"Sqrt-Weights \
[%]\"\>"},
        {
         RowBox[{"C", "[", "V", "]"}], 
         FractionBox[
          RowBox[{"16", " ", 
           SuperscriptBox["m", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "V", "]"}], "2"]}], 
          RowBox[{
           SuperscriptBox["P", "2"], " ", 
           SuperscriptBox["r", "8"], " ", 
           SuperscriptBox["t", "4"]}]], "0.000011474602690917058`", 
         "32.04584238505814`", "32.04584238505814`", "27.153469761961702`"},
        {
         RowBox[{"C", "[", "m", "]"}], 
         FractionBox[
          RowBox[{"16", " ", 
           SuperscriptBox["V", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "m", "]"}], "2"]}], 
          RowBox[{
           SuperscriptBox["P", "2"], " ", 
           SuperscriptBox["r", "8"], " ", 
           SuperscriptBox["t", "4"]}]], "4.583433588575077`*^-6", 
         "12.800442361121924`", "12.800442361121924`", "17.16137084129202`"},
        {
         RowBox[{"C", "[", "r", "]"}], 
         FractionBox[
          RowBox[{"256", " ", 
           SuperscriptBox["m", "2"], " ", 
           SuperscriptBox["V", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "r", "]"}], "2"]}], 
          RowBox[{
           SuperscriptBox["P", "2"], " ", 
           SuperscriptBox["r", "10"], " ", 
           SuperscriptBox["t", "4"]}]], "0.000014814861377365095`", 
         "41.37439225075116`", "41.37439225075116`", "30.85356134047339`"},
        {
         RowBox[{"C", "[", "P", "]"}], 
         FractionBox[
          RowBox[{"16", " ", 
           SuperscriptBox["m", "2"], " ", 
           SuperscriptBox["V", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "P", "]"}], "2"]}], 
          RowBox[{
           SuperscriptBox["P", "4"], " ", 
           SuperscriptBox["r", "8"], " ", 
           SuperscriptBox["t", "4"]}]], "1.6595509367071775`*^-6", 
         "4.634731953707713`", "4.634731953707713`", "10.32646786707303`"},
        {
         RowBox[{"C", "[", "t", "]"}], 
         FractionBox[
          RowBox[{"64", " ", 
           SuperscriptBox["m", "2"], " ", 
           SuperscriptBox["V", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "t", "]"}], "2"]}], 
          RowBox[{
           SuperscriptBox["P", "2"], " ", 
           SuperscriptBox["r", "8"], " ", 
           SuperscriptBox["t", "6"]}]], "3.2743888521170533`*^-6", 
         "9.144591049361066`", "9.144591049361066`", "14.505130189199866`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]}
   },
   AutoDelete->False,
   GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
       GrayLevel[0.85], 
       GrayLevel[1], 
       GrayLevel[0.85]}},
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.3423414853318556`", ",", "0.005983881469889043`"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["Darstellung der gerundeten Form:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RoundedErrorForm", "@", "t7result"}]], "Input"],

Cell[BoxData["\<\"(1.342\[PlusMinus]0.006)\"\>"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Beispiel: A1 - \[CapitalADoubleDot]u\[SZ]erer Photoeffekt: Regression der \
Einsteingrade und Berechnung der Grenzfrequenz\
\>", "Section"],

Cell[CellGroupData[{

Cell["Daten eingeben", "Subsubsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dataA1", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"8.213492", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "2.8118199681*^-19"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"7.402282913580248", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "2.4240932260599998*^-19"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"6.87597380733945", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "2.1068622553*^-19"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"6.093342642276422", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "1.61499403296*^-19"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"5.490704358974359", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "1.276614330816*^-19"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"5.1688354827586206", "*", 
        RowBox[{"10", "^", "14"}]}], ",", "1.1218440693240002*^-19"}], 
      "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dataA1Error", "=", 
   RowBox[{"{", 
    RowBox[{
    "6.248488818*^-21", ",", "2.883917916*^-21", ",", "2.883917916*^-21", ",",
      "6.7291418040000006*^-21", ",", "9.132406734*^-22", ",", 
     "1.441958958*^-21"}], "}"}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Daten mit voller Auswertung regressieren", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FitPlot", "[", 
  RowBox[{"dataA1", ",", "dataA1Error", ",", 
   RowBox[{
    RowBox[{"h", "*", "\[Nu]"}], "+", "Wa"}], ",", "\[Nu]", ",", 
   "\"\<Frequenz \[Nu] [Hz]\>\"", ",", "\"\<Energie E [J]\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"5.15*^14", ",", "2.85*^-19"}], "}"}], ",", 
   RowBox[{"Units", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<J\>\"", ",", "\"\<J s\>\""}], "}"}]}], ",", 
   RowBox[{"FunctionNames", "\[Rule]", 
    RowBox[{"{", "\"\<E(\[Nu])\>\"", "}"}]}], ",", 
   RowBox[{"ParameterNames", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Wa", "\[Rule]", "\"\<\!\(\*SubscriptBox[\(W\), \(a\)]\)\>\""}], 
     "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {
         ItemBox[
          StyleBox["\<\"regression\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         TemplateBox[{"\"polynomial linear\""},
          "LineLegend",
          DisplayFunction->(StyleBox[
            StyleBox[
             PaneBox[
              TagBox[
               GridBox[{{
                  TagBox[
                   GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                GridBoxAlignment -> {
                 "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                False, 
                GridBoxItemSize -> {
                 "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
               "Grid"], Alignment -> Left, AppearanceElements -> None, 
              ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
              "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
            FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
            False]& ),
          Editable->True,
          InterpretationFunction:>(RowBox[{"LineLegend", "[", 
             RowBox[{
               RowBox[{"{", 
                 RowBox[{"Directive", "[", 
                   RowBox[{"{", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.368417, 0.506779, 0.709798]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"Dashing", "[", 
                    RowBox[{"{", "}"}], "]"}]}], "}"}], "]"}], "}"}], ",", 
               RowBox[{"{", #, "}"}]}], "]"}]& )]},
        {
         ItemBox[
          StyleBox["\<\"equation\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         RowBox[{"{", 
          RowBox[{
           StyleBox[
            TagBox[
             FormBox[
              RowBox[{
               RowBox[{"h", " ", "\[Nu]"}], "+", "Wa"}],
              TraditionalForm],
             TraditionalForm,
             Editable->True],
            StripOnInput->False,
            FontColor->RGBColor[1, 0.5, 0]], ",", 
           TagBox[
            FormBox[
             RowBox[{
              RowBox[{"h", " ", "\[Nu]"}], 
              "+", "\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>"}],
             TraditionalForm],
            TraditionalForm,
            Editable->True]}], "}"}]},
        {
         ItemBox[
          StyleBox["\<\"parameters\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         TagBox[GridBox[{
            {
             ItemBox["\<\"\"\>",
              Background->GrayLevel[0.7],
              
              StripOnInput->
               False], "\<\"value\"\>", "\<\"error\"\>", "\<\"p-value\"\>", 
             ItemBox["\<\"\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False]},
            {
             ItemBox["Wa",
              Background->GrayLevel[0.7],
              StripOnInput->False], 
             RowBox[{"-", "1.9074148183059154`*^-19"}], 
             "7.269603308757982`*^-21", "0.00001253774963300167`", 
             ItemBox["\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False]},
            {
             ItemBox["h",
              Background->GrayLevel[0.7],
              StripOnInput->False], "5.8186117508030976`*^-34", 
             "1.2766358257394272`*^-35", "1.3859557534429206`*^-6", 
             ItemBox["\<\"\\!\\(\\*FormBox[\\\"h\\\", TraditionalForm]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False]}
           },
           AutoDelete->False,
           GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
               GrayLevel[1], 
               RGBColor[0.88, 1, 0.88], 
               RGBColor[0.88, 1, 0.88]}},
           GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
           
           GridBoxItemSize->{
            "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
          "Grid"]},
        {
         ItemBox[
          StyleBox["\<\"model-object\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         TagBox[
          RowBox[{"FittedModel", "[", 
           TagBox[
            PanelBox[
             TagBox[
              RowBox[{
               RowBox[{"-", "1.9074148183059154`*^-19"}], "+", 
               RowBox[{"5.8186117508030976`*^-34", " ", "\[Nu]"}]}],
              Short[#, 2]& ],
             FrameMargins->5],
            Editable -> False], "]"}],
          InterpretTemplate[
          FittedModel[{
            "Linear", {-1.9074148183059154`*^-19, 
             5.8186117508030976`*^-34}, {{$CellContext`\[Nu]}, {
              1, $CellContext`\[Nu]}}, {0, 0}}, {{2.561238409427186*^40, 
            1.202359142203318*^41, 1.202359142203318*^41, 
            2.2084147509856857`*^40, 1.1990285074603723`*^42, 
            4.809436568813272*^41}}, {{8.213492*^14, 2.8118199681*^-19}, {
            7.402282913580249*^14, 2.4240932260599998`*^-19}, {
            6.87597380733945*^14, 2.1068622553*^-19}, {6.093342642276422*^14, 
            1.61499403296*^-19}, {5.490704358974359*^14, 
            1.276614330816*^-19}, {5.16883548275862*^14, 
            1.1218440693240002`*^-19}}, {{1., 8.213492*^14}, {1., 
            7.402282913580249*^14}, {1., 6.87597380733945*^14}, {1., 
            6.093342642276422*^14}, {1., 5.490704358974359*^14}, {1., 
            5.16883548275862*^14}}, 
            Function[Null, 
             Internal`LocalizedBlock[{$CellContext`\[Nu]}, #], {HoldAll}]]& ],
          Editable->False,
          SelectWithContents->True,
          Selectable->True]},
        {
         ItemBox[
          
          StyleBox["\<\"{\\!\\(\\*SuperscriptBox[\\(\[Chi]\\), \
\\(2\\)]\\)/dof,\[Alpha]}\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         StyleBox[
          RowBox[{"{", 
           RowBox[{"1.4085739514421411`", ",", "0.8427034615590379`"}], "}"}],
          StripOnInput->False,
          FontColor->RGBColor[0, 0.7, 0]]},
        {
         ItemBox[
          
          StyleBox["\<\"{\\!\\(\\*SuperscriptBox[\\(R\\), \
\\(2\\)]\\),\\!\\(\\*SuperscriptBox[OverscriptBox[\\(R\\), \\(~\\)], \\(2\\)]\
\\)}\\n{\\!\\(\\*SuperscriptBox[\\(R\\), \
\\(2\\)]\\)%,\\!\\(\\*SuperscriptBox[OverscriptBox[\\(R\\), \\(~\\)], \
\\(2\\)]\\)%}\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         StyleBox["\<\"\\!\\(\\*FormBox[RowBox[{\\\"{\\\", \
RowBox[{\\\"0.9980781484656883`\\\", \\\",\\\", \
\\\"0.9975976855821104`\\\"}], \\\"}\\\"}], \
TraditionalForm]\\)\\n\\!\\(\\*FormBox[RowBox[{\\\"{\\\", \
RowBox[{\\\"95.61610728426017`\\\", \\\",\\\", \\\"95.09865894076975`\\\"}], \
\\\"}\\\"}], TraditionalForm]\\)\"\>",
          StripOnInput->False,
          FontColor->RGBColor[1, 0.5, 0]]},
        {
         ItemBox[
          StyleBox["\<\"correlations\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         TagBox[GridBox[{
            {"\<\"\"\>", 
             ItemBox["Wa",
              Background->GrayLevel[0.7],
              StripOnInput->False], 
             ItemBox["h",
              Background->GrayLevel[0.7],
              StripOnInput->False], "\<\"\"\>"},
            {"Wa", "1.0000000000000002`", 
             RowBox[{
             "-", "0.9932056170487095`"}], \
"\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>"},
            {"h", 
             RowBox[{"-", "0.9932056170487095`"}], 
             "0.9999999999999999`", "\<\"\\!\\(\\*FormBox[\\\"h\\\", \
TraditionalForm]\\)\"\>"},
            {"\<\"\"\>", 
             ItemBox["\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False], 
             ItemBox["\<\"\\!\\(\\*FormBox[\\\"h\\\", TraditionalForm]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False], "\<\"\"\>"}
           },
           AutoDelete->False,
           GridBoxBackground->{"Columns" -> {
               GrayLevel[0.7], 
               GrayLevel[1], 
               GrayLevel[1], 
               GrayLevel[0.7]}, "Rows" -> {{None}}},
           GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
           GridBoxItemSize->{
            "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
          "Grid"]},
        {
         ItemBox[
          StyleBox["\<\"covariances\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         TagBox[GridBox[{
            {"\<\"\"\>", 
             ItemBox["Wa",
              Background->GrayLevel[0.7],
              StripOnInput->False], 
             ItemBox["h",
              Background->GrayLevel[0.7],
              StripOnInput->False], "\<\"\"\>"},
            {"Wa", "5.284713226670501`*^-41", 
             RowBox[{
             "-", "9.217579827703368`*^-56"}], \
"\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>"},
            {"h", 
             RowBox[{"-", "9.21757982770337`*^-56"}], 
             "1.6297990315613892`*^-70", "\<\"\\!\\(\\*FormBox[\\\"h\\\", \
TraditionalForm]\\)\"\>"},
            {"\<\"\"\>", 
             ItemBox["\<\"\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False], 
             ItemBox["\<\"\\!\\(\\*FormBox[\\\"h\\\", TraditionalForm]\\)\"\>",
              Background->GrayLevel[0.7],
              StripOnInput->False], "\<\"\"\>"}
           },
           AutoDelete->False,
           GridBoxBackground->{"Columns" -> {
               GrayLevel[0.7], 
               GrayLevel[1], 
               GrayLevel[1], 
               GrayLevel[0.7]}, "Rows" -> {{None}}},
           GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
           
           GridBoxItemSize->{
            "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
          "Grid"]},
        {
         ItemBox[
          StyleBox["\<\"residuals\"\>",
           StripOnInput->False,
           FontColor->GrayLevel[1]],
          Background->GrayLevel[0.4],
          StripOnInput->False], 
         GraphicsBox[{{}, {{}, {}, 
            {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
             0.019444444444444445`], AbsoluteThickness[1.6], 
             LineBox[{{1., -5.987732022680818*^-21}, {2., 
              2.4407009993212903`*^-21}, {3., 1.3414874245951984`*^-21}, {
              4., -2.307065873600325*^-21}, {5., -1.079854120948373*^-21}, {
              6., 2.17141998351848*^-21}}]}}, {}, {}, {{}, {}}},
          AspectRatio->NCache[
            Rational[1, 5], 0.2],
          Axes->{True, True},
          AxesLabel->{None, None},
          AxesOrigin->{0, 0},
          DisplayFunction->Identity,
          Frame->{{False, False}, {False, False}},
          FrameLabel->{{None, None}, {None, None}},
          FrameTicks->{{False, False}, {False, False}},
          GridLines->{None, None},
          GridLinesStyle->Directive[
            GrayLevel[0.5, 0.4]],
          ImagePadding->1,
          ImageSize->250,
          Method->{"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                (
                Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
                 Part[#, 1]], 
                (
                Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
                 Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                (
                Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
                 Part[#, 1]], 
                (
                Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
                 Part[#, 2]]}& )}},
          
          PlotRange->{{0, 7}, {-5.987732022680818*^-21, 
            5.987732022680818*^-21}},
          PlotRangeClipping->True,
          PlotRangePadding->{{0, 0}, {0, 0}},
          Ticks->{{}, {}}]}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"], 
     GraphicsBox[{{{{}, {}, 
         {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
          Opacity[1.], Dashing[{}], CapForm["Butt"], LineBox[CompressedData["
1:eJwVz2lQk2cUBeDsX4IEQpK31VhkLYhb1RpRcsvyXdTBBTRBUKxLIpmypKJG
BU1lk0UUwYmAgxqVcRlBqNCq6EAUxaV1QCGhIGmVYhFtpbgVHK0kTX+cOfP8
O8dHk6bUshgMRpQz/7fpcotsz8Qxmmrrr9x+2qEYzMxddMZrjPYt+ffAm3qH
omK8+WrE3DG64MLVcnrEoRhdMvv4ro1jdND7I8rXoQzQ2ZTvC2+P0e3DKrW7
lQHzhjqfER87/UA4ofbrtUx4oHknebjLTjPzPujfDjBhIlNlkf9mpwV14xX8
DBaYfYX+0mAHneagQ5cy2UB0Y1RJpYOu49SEu1WwYZ6sLAg+OOgJ6uktEyZz
4OGZyDT5FAY+HWw2xNZzIE1pM8rWMLDQq2iuMJILDeFejJa9DCzKUU8faefC
anVwcdYlBjaFxFUFrOTBr+Up/a6DDGw9WyjtfsYDvuuy7AQJE4PgduvFzRSI
KhY/3x7JRG1e2FYPLh8ylRPTzm9jotWv29RYwYeK5UfDqqqYaBrK4B6YJIAi
zdzeRAsTv0papfu9QQCBl7ObnzuYGCaMdN+ocIGTnoNJIbNYSLS6ho0dLpDV
OGnOWTULPaUNHXWaccA/xIuylLAwZ4g5LfDFONDlCgLbrrPQnF5aMCnLFQw/
NkUX/M3C+GlBngsEQnj1SUR5goyNfvP6Zl46KISwkv7THxezsXW0E0/5ucH8
RbPs6zKcftxhbKl1AzfpMe/SajYe3unavybCHaIkNnVvLxtPpCbWhlvd4YvY
u0N/8DiYhwOra6aL4OW5B+HNwRw8WTNnrT5dBAlhhY9jtRzUBey3q5pFMJUv
PkoZOWhpT1vX6xBBz0Cj996bHDx9/43ZM8oDZrCFr356yUH/AUPTzgMesF6+
e2+PNxfdVKeulPV4gKL6WMrZaC6e6+5Q3f1MDLtjSlNLDVy05RfXHd8ghiMC
k3xhLRczSgZ58hoxaLgJUeZeLt45v9D+ZFgMHDI7gDWOh5m+C0Zi5kvANf6R
j1rBw/u3OmvUuRK4bgv+JjvJaUOifd8dCSzbWrf920oehhSXr9CLpJC/xNIu
u8tDZUMbrV8pBb/4e5FP3vKQxzK++bRSCsd2PMpL96fQt3akyvBECi09J9za
V1D4KmX9y9EAAtWH6wyHlRQebwu4MnkyAWN803O1isKlM4azE4IIaHu6b47G
Unju9Xfia1MJuPQI071XUbgh44g8fyaBld2GPv06Ci35vxgkIQT+6oqvl6VS
+MPJJfyZywh0lWk9nzqtYUk61dEEzLH6fRd0FIoSbZWHYggc7CpJjNxEoS4w
eeq7FQTkXbfHb9pCof/3BdHX4ghkW7/MuZFOYUXTjbKlGwgkH4oY3p9BYaRn
0dpMNQGlKmZN3E7nn8zlAfUaAp9bU+QvdlEYTfc1SrQE7lmq/pRmUsj7+aPN
lkzgovFCXJ/TF6fcOuWaSsCkNLdWZzn3Fe/XheoIbLY8NIXmUHgtRmav2kRg
tXHQRZDr3NfQf8eaRoBW/pNudVomqS7lbiEwTcx6atpD4b1tm1cFbyVALO7K
pDwKd3QH+yTrCfwH3bHelA==
           "]]}}, {}, {}}, {{}, 
        GraphicsComplexBox[{{8.213492*^14, 2.8118199681*^-19}, {
         7.402282913580249*^14, 2.4240932260599998`*^-19}, {
         6.87597380733945*^14, 2.1068622553*^-19}, {6.093342642276422*^14, 
         1.61499403296*^-19}, {5.490704358974359*^14, 1.276614330816*^-19}, {
         5.16883548275862*^14, 1.1218440693240002`*^-19}, {8.213492*^14, 
         2.8118199681*^-19}, {7.402282913580249*^14, 
         2.4240932260599998`*^-19}, {6.87597380733945*^14, 
         2.1068622553*^-19}, {6.093342642276422*^14, 1.61499403296*^-19}, {
         5.490704358974359*^14, 1.276614330816*^-19}, {5.16883548275862*^14, 
         1.1218440693240002`*^-19}}, {
          {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
           AbsoluteThickness[1.6], CapForm["Butt"], {InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], Offset[{-2.9484457662642556`, -1.7022859568436806`}, {
                   0, 0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}],
                   Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 7], {
             LineBox[{{8.213492*^14, 2.8743048562800003`*^-19}, {8.213492*^14,
               2.74933507992*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {8.213492*^14, 2.8743048562800003`*^-19}], 
               Offset[{-1.5, 0}, {8.213492*^14, 2.8743048562800003`*^-19}]}], 
             LineBox[{
              Offset[{1.5, 0}, {8.213492*^14, 2.74933507992*^-19}], 
               Offset[{-1.5, 0}, {8.213492*^14, 2.74933507992*^-19}]}]}}, {
            InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], Offset[{-2.9484457662642556`, -1.7022859568436806`}, {
                   0, 0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}],
                   Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 8], {
             LineBox[{{7.402282913580249*^14, 2.4529324052199996`*^-19}, {
              7.402282913580249*^14, 2.3952540469*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {7.402282913580249*^14, 
                2.4529324052199996`*^-19}], 
               Offset[{-1.5, 0}, {7.402282913580249*^14, 
                2.4529324052199996`*^-19}]}], 
             LineBox[{
              Offset[{1.5, 0}, {7.402282913580249*^14, 2.3952540469*^-19}], 
               Offset[{-1.5, 0}, {7.402282913580249*^14, 
                2.3952540469*^-19}]}]}}, {InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], 
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], Offset[{-3.2885639388890184`, -0.8811680516834245}, {
                   0, 0}], Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 9], {
             LineBox[{{6.87597380733945*^14, 2.1357014344600001`*^-19}, {
              6.87597380733945*^14, 2.07802307614*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {6.87597380733945*^14, 
                2.1357014344600001`*^-19}], 
               Offset[{-1.5, 0}, {6.87597380733945*^14, 
                2.1357014344600001`*^-19}]}], 
             LineBox[{
              Offset[{1.5, 0}, {6.87597380733945*^14, 2.07802307614*^-19}], 
               Offset[{-1.5, 0}, {6.87597380733945*^14, 
                2.07802307614*^-19}]}]}}, {InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], Offset[{-2.9484457662642556`, -1.7022859568436806`}, {
                   0, 0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}],
                   Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 10], {
             LineBox[{{6.093342642276422*^14, 1.682285451*^-19}, {
              6.093342642276422*^14, 1.54770261492*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {6.093342642276422*^14, 1.682285451*^-19}], 
               Offset[{-1.5, 0}, {6.093342642276422*^14, 1.682285451*^-19}]}],
              LineBox[{
              Offset[{1.5, 0}, {6.093342642276422*^14, 1.54770261492*^-19}], 
               Offset[{-1.5, 0}, {6.093342642276422*^14, 
                1.54770261492*^-19}]}]}}, {InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], Offset[{-2.9484457662642556`, -1.7022859568436806`}, {
                   0, 0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}],
                   Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 11], {
             LineBox[{{5.490704358974359*^14, 1.2857467375499998`*^-19}, {
              5.490704358974359*^14, 1.267481924082*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {5.490704358974359*^14, 
                1.2857467375499998`*^-19}], 
               Offset[{-1.5, 0}, {5.490704358974359*^14, 
                1.2857467375499998`*^-19}]}], 
             LineBox[{
              Offset[{1.5, 0}, {5.490704358974359*^14, 1.267481924082*^-19}], 
               Offset[{-1.5, 0}, {5.490704358974359*^14, 
                1.267481924082*^-19}]}]}}, {InsetBox[
             GraphicsBox[
              {RGBColor[0.368417, 0.506779, 0.709798], AbsolutePointSize[6], 
               AbsoluteThickness[1.6], CapForm["Butt"], 
               DynamicBox[Typeset`ToBoxes[
                 EdgeForm[
                  Directive[
                   RGBColor[0.368417, 0.506779, 0.709798], 
                   JoinForm["Round"], 
                   AbsoluteThickness[1], 
                   Opacity[1]]], StandardForm, Graphics],
                
                ImageSizeCache->{{
                 4.503599627370496*^15, -4.503599627370496*^15}, {
                 4.503599627370496*^15, -4.503599627370496*^15}}], 
               {FaceForm[{RGBColor[0.368417, 0.506779, 0.709798], Opacity[
                0.15]}], 
                PolygonBox[{
                 Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}],
                   Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}],
                   Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}],
                   Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], Offset[{-2.407395887205594, -2.407395887205594}, {0, 
                   0}], Offset[{-2.9484457662642556`, -1.7022859568436806`}, {
                   0, 0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}],
                   Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}],
                   Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}],
                   Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}},
              AlignmentPoint->{0, 0}], 12], {
             LineBox[{{5.16883548275862*^14, 1.1362636589040001`*^-19}, {
              5.16883548275862*^14, 1.1074244797440003`*^-19}}], 
             LineBox[{
              Offset[{1.5, 0}, {5.16883548275862*^14, 
                1.1362636589040001`*^-19}], 
               Offset[{-1.5, 0}, {5.16883548275862*^14, 
                1.1362636589040001`*^-19}]}], 
             LineBox[{
              Offset[{1.5, 0}, {5.16883548275862*^14, 
                1.1074244797440003`*^-19}], 
               Offset[{-1.5, 0}, {5.16883548275862*^14, 
                1.1074244797440003`*^-19}]}]}}}, {}}], {}, {}, {{}, {}}}},
      AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
      Axes->{False, False},
      AxesLabel->{None, None},
      AxesOrigin->{5.1079423524137925`*^14, 1.0720868722132803`*^-19},
      DisplayFunction->Identity,
      Epilog->{
        InsetBox[
         BoxData[
          FormBox[
           FrameBox[
            TagBox[
             GridBox[{{
                
                TemplateBox[{
                 "\"E(\[Nu])=\\!\\(\\*FormBox[RowBox[{RowBox[{\\\"h\\\", \\\" \
\\\", \\\"\[Nu]\\\"}], \\\"+\\\", \
\\\"\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\*SubscriptBox[\\\\\\\\(W\\\\\\\\), \\\\\
\\\\(a\\\\\\\\)]\\\\\\\\)\\\\\\\"\\\"}], \
TraditionalForm]\\)\\n\\!\\(\\*SuperscriptBox[\\(\[Chi]\\), \
\\(2\\)]\\)/dof=\\!\\(\\*FormBox[\\\"1.4085739514421411`\\\", \
TraditionalForm]\\)\\n\\!\\(\\*SuperscriptBox[OverscriptBox[\\(R\\), \
\\(_\\)], \\(2\\)]\\)=\\!\\(\\*FormBox[\\\"0.9975976855821104`\\\", \
TraditionalForm]\\)\\n\\!\\(\\*SubscriptBox[\\(W\\), \\(a\\)]\\)=(-1.91\
\[PlusMinus]0.08)\\!\\(\\*FormBox[TemplateBox[List[\\\"10\\\", RowBox[List[\\\
\"-\\\", \\\"19\\\"]]], \\\"Superscript\\\", Rule[SyntaxForm, \
SuperscriptBox]], TraditionalForm]\\) J\\n\\!\\(\\*FormBox[\\\"h\\\", \
TraditionalForm]\\)=(5.82\[PlusMinus]0.13)\\!\\(\\*FormBox[TemplateBox[List[\\\
\"10\\\", RowBox[List[\\\"-\\\", \\\"34\\\"]]], \\\"Superscript\\\", \
Rule[SyntaxForm, SuperscriptBox]], TraditionalForm]\\) J s\""}, "LineLegend", 
                 DisplayFunction -> (FormBox[
                   StyleBox[
                    StyleBox[
                    PaneBox[
                    TagBox[
                    GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {
                    LineBox[{{0, 25}, {60, 25}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {
                    InsetBox[
                    GraphicsBox[{
                    DynamicBox[
                    Typeset`ToBoxes[
                    EdgeForm[
                    Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], TraditionalForm, Graphics]], 
                    FaceForm[
                    Directive[
                    Opacity[0.15], 
                    RGBColor[0.368417, 0.506779, 0.709798]]], 
                    PolygonBox[{
                    Offset[{0, 3.404571913687361}, {0, 0}], 
                    Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                    Offset[{3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    Offset[{0, -3.404571913687361}, {0, 0}], 
                    
                    Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    
                    Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    
                    Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    Offset[{-3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                    Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                    0}]}]}, {DefaultBaseStyle -> {"Graphics", {
                    AbsolutePointSize[6]}, 
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]]}, AlignmentPoint -> {0, 0}}], 
                    NCache[
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}], 
                    Scaled[{0.5, 0.5}]], Automatic, 
                    Scaled[1]]}}}, AspectRatio -> Full, ImageSize -> {60, 25},
                     PlotRangePadding -> None, ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.29200000000000004`] -> 
                    Baseline)], #}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                    False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                    GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}],
                     "Grid"], Alignment -> Left, AppearanceElements -> None, 
                    ImageMargins -> {{10, 0}, {20, 10}}, ImageSizeAction -> 
                    "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
                    FontSize -> 13, FontFamily -> "Arial"}, Background -> 
                    Automatic, StripOnInput -> False], TraditionalForm]& ), 
                 InterpretationFunction :> (RowBox[{"LineLegend", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> 
                    RGBColor[0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.368417, 0.506779, 0.709798]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"Dashing", "[", 
                    RowBox[{"{", "}"}], "]"}]}], "}"}], "]"}], "}"}], ",", 
                    RowBox[{"{", #, "}"}], ",", 
                    RowBox[{"LegendMarkers", "\[Rule]", 
                    RowBox[{"{", 
                    GraphicsBox[{
                    DynamicBox[
                    Typeset`ToBoxes[
                    EdgeForm[
                    Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                    FaceForm[
                    Directive[
                    Opacity[0.15], 
                    RGBColor[0.368417, 0.506779, 0.709798]]], 
                    PolygonBox[{
                    Offset[{0, 3.404571913687361}, {0, 0}], 
                    Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                    Offset[{3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    Offset[{0, -3.404571913687361}, {0, 0}], 
                    
                    Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    
                    Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    
                    Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    Offset[{-3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                    0}]}]}, AlignmentPoint -> {0, 0}], "}"}]}], ",", 
                    RowBox[{"LegendMarkerSize", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"60", ",", "25"}], "}"}]}], ",", 
                    RowBox[{"LegendMargins", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"10", ",", "0"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"20", ",", "10"}], "}"}]}], "}"}]}], ",", 
                    RowBox[{"LabelStyle", "\[Rule]", "13"}]}], "]"}]& ), 
                 Editable -> True]}, {""}}, AutoDelete -> False, 
              GridBoxItemSize -> {
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"],
             Background -> Opacity[0.8, 
              GrayLevel[1]], FrameStyle -> GrayLevel[0.5], StripOnInput -> 
            False], TraditionalForm]], {5.15*^14, 2.85*^-19}, {Left, Top}]},
      Frame->{{True, True}, {True, True}},
      FrameLabel->{
        FormBox[
         StyleBox[
         "\"Frequenz \[Nu] [Hz]\"", {
          FontSize -> 16, FontTracking -> "Extended"}, StripOnInput -> False],
          TraditionalForm], 
        FormBox[
         StyleBox[
         "\"Energie E [J]\"", {FontSize -> 16, FontTracking -> "Extended"}, 
          StripOnInput -> False], TraditionalForm]},
      FrameStyle->Automatic,
      FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
      FrameTicksStyle->14,
      GridLines->None,
      GridLinesStyle->Directive[
        GrayLevel[0.5, 0.4]],
      ImagePadding->All,
      ImageSize->Large,
      LabelStyle->{FontFamily -> "Times"},
      Method->{
       "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
        AbsolutePointSize[6], "ScalingFunctions" -> None, 
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
            (Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
             Part[#, 1]], 
            (Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
             Part[#, 2]]}& ), "CopiedValueFunction" -> ({
            (Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
             Part[#, 1]], 
            (Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
             Part[#, 2]]}& )}},
      PlotRange->{{5.1079423524137925`*^14, 8.274385130344828*^14}, {
       1.0720868722132803`*^-19, 2.9096424638107205`*^-19}},
      PlotRangeClipping->True,
      PlotRangePadding->{{0, 0}, {0, 0}},
      Ticks->{Automatic, Automatic}]},
    {
     StyleBox["\<\"not saved (no file specified)\"\>",
      StripOnInput->False,
      FontColor->RGBColor[1, 0.5, 0]], ""}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Daten regressieren und nur das wichtige zur\[UDoubleDot]ckgeben", \
"Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fit1Result", "=", 
  RowBox[{"FitPlot", "[", 
   RowBox[{"dataA1", ",", "dataA1Error", ",", 
    RowBox[{
     RowBox[{"h", "*", "\[Nu]"}], "+", "Wa"}], ",", "\[Nu]", ",", 
    RowBox[{"Analyze", "\[Rule]", "False"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    TagBox[
     RowBox[{"FittedModel", "[", 
      TagBox[
       PanelBox[
        TagBox[
         RowBox[{
          RowBox[{"-", "1.9074148183059154`*^-19"}], "+", 
          RowBox[{"5.8186117508030976`*^-34", " ", "\[Nu]"}]}],
         Short[#, 2]& ],
        FrameMargins->5],
       Editable -> False], "]"}],
     InterpretTemplate[
     FittedModel[{
       "Linear", {-1.9074148183059154`*^-19, 
        5.8186117508030976`*^-34}, {{$CellContext`\[Nu]}, {
         1, $CellContext`\[Nu]}}, {0, 0}}, {{2.561238409427186*^40, 
       1.202359142203318*^41, 1.202359142203318*^41, 2.2084147509856857`*^40, 
       1.1990285074603723`*^42, 4.809436568813272*^41}}, {{8.213492*^14, 
       2.8118199681*^-19}, {7.402282913580249*^14, 
       2.4240932260599998`*^-19}, {6.87597380733945*^14, 2.1068622553*^-19}, {
       6.093342642276422*^14, 1.61499403296*^-19}, {5.490704358974359*^14, 
       1.276614330816*^-19}, {5.16883548275862*^14, 
       1.1218440693240002`*^-19}}, {{1., 8.213492*^14}, {1., 
       7.402282913580249*^14}, {1., 6.87597380733945*^14}, {1., 
       6.093342642276422*^14}, {1., 5.490704358974359*^14}, {1., 
       5.16883548275862*^14}}, 
       Function[Null, 
        Internal`LocalizedBlock[{$CellContext`\[Nu]}, #], {HoldAll}]]& ],
     Editable->False,
     SelectWithContents->True,
     Selectable->True], "}"}], ",", 
   TemplateBox[{GraphicsBox[{{{{{}, {}, {
           Directive[
            Opacity[1.], 
            CapForm["Butt"], 
            AbsoluteThickness[1.6], 
            RGBColor[0.368417, 0.506779, 0.709798], 
            Dashing[{}]], 
           LineBox[CompressedData["
1:eJwVz2lQk2cUBeDsX4IEQpK31VhkLYhb1RpRcsvyXdTBBTRBUKxLIpmypKJG
BU1lk0UUwYmAgxqVcRlBqNCq6EAUxaV1QCGhIGmVYhFtpbgVHK0kTX+cOfP8
O8dHk6bUshgMRpQz/7fpcotsz8Qxmmrrr9x+2qEYzMxddMZrjPYt+ffAm3qH
omK8+WrE3DG64MLVcnrEoRhdMvv4ro1jdND7I8rXoQzQ2ZTvC2+P0e3DKrW7
lQHzhjqfER87/UA4ofbrtUx4oHknebjLTjPzPujfDjBhIlNlkf9mpwV14xX8
DBaYfYX+0mAHneagQ5cy2UB0Y1RJpYOu49SEu1WwYZ6sLAg+OOgJ6uktEyZz
4OGZyDT5FAY+HWw2xNZzIE1pM8rWMLDQq2iuMJILDeFejJa9DCzKUU8faefC
anVwcdYlBjaFxFUFrOTBr+Up/a6DDGw9WyjtfsYDvuuy7AQJE4PgduvFzRSI
KhY/3x7JRG1e2FYPLh8ylRPTzm9jotWv29RYwYeK5UfDqqqYaBrK4B6YJIAi
zdzeRAsTv0papfu9QQCBl7ObnzuYGCaMdN+ocIGTnoNJIbNYSLS6ho0dLpDV
OGnOWTULPaUNHXWaccA/xIuylLAwZ4g5LfDFONDlCgLbrrPQnF5aMCnLFQw/
NkUX/M3C+GlBngsEQnj1SUR5goyNfvP6Zl46KISwkv7THxezsXW0E0/5ucH8
RbPs6zKcftxhbKl1AzfpMe/SajYe3unavybCHaIkNnVvLxtPpCbWhlvd4YvY
u0N/8DiYhwOra6aL4OW5B+HNwRw8WTNnrT5dBAlhhY9jtRzUBey3q5pFMJUv
PkoZOWhpT1vX6xBBz0Cj996bHDx9/43ZM8oDZrCFr356yUH/AUPTzgMesF6+
e2+PNxfdVKeulPV4gKL6WMrZaC6e6+5Q3f1MDLtjSlNLDVy05RfXHd8ghiMC
k3xhLRczSgZ58hoxaLgJUeZeLt45v9D+ZFgMHDI7gDWOh5m+C0Zi5kvANf6R
j1rBw/u3OmvUuRK4bgv+JjvJaUOifd8dCSzbWrf920oehhSXr9CLpJC/xNIu
u8tDZUMbrV8pBb/4e5FP3vKQxzK++bRSCsd2PMpL96fQt3akyvBECi09J9za
V1D4KmX9y9EAAtWH6wyHlRQebwu4MnkyAWN803O1isKlM4azE4IIaHu6b47G
Unju9Xfia1MJuPQI071XUbgh44g8fyaBld2GPv06Ci35vxgkIQT+6oqvl6VS
+MPJJfyZywh0lWk9nzqtYUk61dEEzLH6fRd0FIoSbZWHYggc7CpJjNxEoS4w
eeq7FQTkXbfHb9pCof/3BdHX4ghkW7/MuZFOYUXTjbKlGwgkH4oY3p9BYaRn
0dpMNQGlKmZN3E7nn8zlAfUaAp9bU+QvdlEYTfc1SrQE7lmq/pRmUsj7+aPN
lkzgovFCXJ/TF6fcOuWaSsCkNLdWZzn3Fe/XheoIbLY8NIXmUHgtRmav2kRg
tXHQRZDr3NfQf8eaRoBW/pNudVomqS7lbiEwTcx6atpD4b1tm1cFbyVALO7K
pDwKd3QH+yTrCfwH3bHelA==
            "]]}}}, {}, {}}, {{}, 
        GraphicsComplexBox[{{8.213492*^14, 2.8118199681*^-19}, {
         7.402282913580249*^14, 2.4240932260599998`*^-19}, {
         6.87597380733945*^14, 2.1068622553*^-19}, {6.093342642276422*^14, 
         1.61499403296*^-19}, {5.490704358974359*^14, 1.276614330816*^-19}, {
         5.16883548275862*^14, 1.1218440693240002`*^-19}, {8.213492*^14, 
         2.8118199681*^-19}, {7.402282913580249*^14, 
         2.4240932260599998`*^-19}, {6.87597380733945*^14, 
         2.1068622553*^-19}, {6.093342642276422*^14, 1.61499403296*^-19}, {
         5.490704358974359*^14, 1.276614330816*^-19}, {5.16883548275862*^14, 
         1.1218440693240002`*^-19}}, {{{
            Directive[
             PointSize[0.012833333333333334`], 
             CapForm["Butt"], 
             AbsoluteThickness[1.6], 
             RGBColor[0.368417, 0.506779, 0.709798], 
             AbsolutePointSize[6]], {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 7], {
              
              LineBox[{{8.213492*^14, 2.8743048562800003`*^-19}, {
               8.213492*^14, 2.74933507992*^-19}}], 
              LineBox[{
                Offset[{1.5, 0}, {8.213492*^14, 2.8743048562800003`*^-19}], 
                Offset[{-1.5, 0}, {8.213492*^14, 2.8743048562800003`*^-19}]}], 
              LineBox[{
                Offset[{1.5, 0}, {8.213492*^14, 2.74933507992*^-19}], 
                Offset[{-1.5, 0}, {8.213492*^14, 2.74933507992*^-19}]}]}}, {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 8], {
              
              LineBox[{{7.402282913580249*^14, 2.4529324052199996`*^-19}, {
               7.402282913580249*^14, 2.3952540469*^-19}}], 
              LineBox[{
                
                Offset[{1.5, 0}, {7.402282913580249*^14, 
                 2.4529324052199996`*^-19}], 
                
                Offset[{-1.5, 0}, {7.402282913580249*^14, 
                 2.4529324052199996`*^-19}]}], 
              LineBox[{
                Offset[{1.5, 0}, {7.402282913580249*^14, 2.3952540469*^-19}], 
                
                Offset[{-1.5, 0}, {7.402282913580249*^14, 
                 2.3952540469*^-19}]}]}}, {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 9], {
              
              LineBox[{{6.87597380733945*^14, 2.1357014344600001`*^-19}, {
               6.87597380733945*^14, 2.07802307614*^-19}}], 
              LineBox[{
                
                Offset[{1.5, 0}, {6.87597380733945*^14, 
                 2.1357014344600001`*^-19}], 
                
                Offset[{-1.5, 0}, {6.87597380733945*^14, 
                 2.1357014344600001`*^-19}]}], 
              LineBox[{
                Offset[{1.5, 0}, {6.87597380733945*^14, 2.07802307614*^-19}], 
                
                Offset[{-1.5, 0}, {6.87597380733945*^14, 
                 2.07802307614*^-19}]}]}}, {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 10], {
              
              LineBox[{{6.093342642276422*^14, 1.682285451*^-19}, {
               6.093342642276422*^14, 1.54770261492*^-19}}], 
              LineBox[{
                Offset[{1.5, 0}, {6.093342642276422*^14, 1.682285451*^-19}], 
                
                Offset[{-1.5, 0}, {6.093342642276422*^14, 
                 1.682285451*^-19}]}], 
              LineBox[{
                Offset[{1.5, 0}, {6.093342642276422*^14, 1.54770261492*^-19}], 
                
                Offset[{-1.5, 0}, {6.093342642276422*^14, 
                 1.54770261492*^-19}]}]}}, {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 11], {
              
              LineBox[{{5.490704358974359*^14, 1.2857467375499998`*^-19}, {
               5.490704358974359*^14, 1.267481924082*^-19}}], 
              LineBox[{
                
                Offset[{1.5, 0}, {5.490704358974359*^14, 
                 1.2857467375499998`*^-19}], 
                
                Offset[{-1.5, 0}, {5.490704358974359*^14, 
                 1.2857467375499998`*^-19}]}], 
              LineBox[{
                
                Offset[{1.5, 0}, {5.490704358974359*^14, 
                 1.267481924082*^-19}], 
                
                Offset[{-1.5, 0}, {5.490704358974359*^14, 
                 1.267481924082*^-19}]}]}}, {
             InsetBox[
              GraphicsBox[{
                Directive[
                 PointSize[0.012833333333333334`], 
                 CapForm["Butt"], 
                 AbsoluteThickness[1.6], 
                 RGBColor[0.368417, 0.506779, 0.709798], 
                 AbsolutePointSize[6]], 
                DynamicBox[
                 Typeset`ToBoxes[
                  EdgeForm[
                   Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                FaceForm[
                 Directive[
                  Opacity[0.15], 
                  RGBColor[0.368417, 0.506779, 0.709798]]], 
                PolygonBox[{
                  Offset[{0, 3.404571913687361}, {0, 0}], 
                  Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                  Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{3.404571913687361, 0}, {0, 0}], 
                  Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 0}], 
                  Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                  Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 0}], 
                  Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  Offset[{0, -3.404571913687361}, {0, 0}], 
                  Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 0}], 
                  
                  Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                   0}], 
                  Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                  
                  Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                   0}], 
                  Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 0}], 
                  Offset[{-3.404571913687361, 0}, {0, 0}], 
                  Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                  Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 0}], 
                  Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                  Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 0}], 
                  
                  Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                   0}]}]}, AlignmentPoint -> {0, 0}], 12], {
              
              LineBox[{{5.16883548275862*^14, 1.1362636589040001`*^-19}, {
               5.16883548275862*^14, 1.1074244797440003`*^-19}}], 
              LineBox[{
                
                Offset[{1.5, 0}, {5.16883548275862*^14, 
                 1.1362636589040001`*^-19}], 
                
                Offset[{-1.5, 0}, {5.16883548275862*^14, 
                 1.1362636589040001`*^-19}]}], 
              LineBox[{
                
                Offset[{1.5, 0}, {5.16883548275862*^14, 
                 1.1074244797440003`*^-19}], 
                
                Offset[{-1.5, 0}, {5.16883548275862*^14, 
                 1.1074244797440003`*^-19}]}]}}}, {}}}], {}, {}, {{}, {}}}}, {
      FrameLabel -> {
         FormBox[
          StyleBox[
          "\"\"", {FontSize -> 16, FontTracking -> "Extended"}, StripOnInput -> 
           False], TraditionalForm], 
         FormBox[
          StyleBox[
          "\"\"", {FontSize -> 16, FontTracking -> "Extended"}, StripOnInput -> 
           False], TraditionalForm]}, GridLines -> None, FrameTicksStyle -> 
       14, DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
       AxesOrigin -> {5.1079423524137925`*^14, 1.0720868722132803`*^-19}, 
       FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
       GridLines -> {{0}, {0}}, DisplayFunction -> Identity, 
       PlotRangePadding -> {{0, 0}, {0, 0}}, PlotRangeClipping -> True, 
       ImagePadding -> All, DisplayFunction -> Identity, AspectRatio -> 
       NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
       AxesLabel -> {None, None}, 
       AxesOrigin -> {5.1079423524137925`*^14, 1.0720868722132803`*^-19}, 
       DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
       FrameLabel -> {{None, None}, {None, None}}, FrameStyle -> Automatic, 
       FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
       GridLines -> {{0}, {0}}, GridLinesStyle -> Directive[
         GrayLevel[0.5, 0.4]], ImageSize -> Large, 
       LabelStyle -> {FontFamily -> "Times"}, 
       Method -> {
        "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
         AbsolutePointSize[6], "ScalingFunctions" -> None, 
         "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
             (Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
              Part[#, 1]], 
             (Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
              Part[#, 2]]}& ), "CopiedValueFunction" -> ({
             (Part[{{Identity, Identity}, {Identity, Identity}}, 1, 2][#]& )[
              Part[#, 1]], 
             (Part[{{Identity, Identity}, {Identity, Identity}}, 2, 2][#]& )[
              Part[#, 2]]}& )}}, 
       PlotRange -> {{5.1079423524137925`*^14, 8.274385130344828*^14}, {
        1.0720868722132803`*^-19, 2.9096424638107205`*^-19}}, 
       PlotRangeClipping -> True, 
       PlotRangePadding -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
       Ticks -> {Automatic, Automatic}}],FormBox[
      FormBox[
       TagBox[
        GridBox[{{
           
           TemplateBox[{
            "\"\\!\\(\\*FormBox[RowBox[{RowBox[{\\\"h\\\", \\\" \\\", \\\"\
\[Nu]\\\"}], \\\"+\\\", \\\"Wa\\\"}], \
TraditionalForm]\\)\\n\\!\\(\\*SuperscriptBox[\\(\[Chi]\\), \
\\(2\\)]\\)/dof=\\!\\(\\*FormBox[\\\"1.4085739514421411`\\\", \
TraditionalForm]\\)\\n\\!\\(\\*SuperscriptBox[OverscriptBox[\\(R\\), \
\\(_\\)], \\(2\\)]\\)=\\!\\(\\*FormBox[\\\"0.9975976855821104`\\\", \
TraditionalForm]\\)\\n\\!\\(\\*FormBox[\\\"Wa\\\", TraditionalForm]\\)=(-1.91\
\[PlusMinus]0.08)\\!\\(\\*FormBox[TemplateBox[List[\\\"10\\\", RowBox[List[\\\
\"-\\\", \\\"19\\\"]]], \\\"Superscript\\\", Rule[SyntaxForm, \
SuperscriptBox]], TraditionalForm]\\)\\n\\!\\(\\*FormBox[\\\"h\\\", \
TraditionalForm]\\)=(5.82\[PlusMinus]0.13)\\!\\(\\*FormBox[TemplateBox[List[\\\
\"10\\\", RowBox[List[\\\"-\\\", \\\"34\\\"]]], \\\"Superscript\\\", \
Rule[SyntaxForm, SuperscriptBox]], TraditionalForm]\\)\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {
                    LineBox[{{0, 25}, {60, 25}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]], {
                    InsetBox[
                    GraphicsBox[{
                    DynamicBox[
                    Typeset`ToBoxes[
                    EdgeForm[
                    Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], TraditionalForm, Graphics]], 
                    FaceForm[
                    Directive[
                    Opacity[0.15], 
                    RGBColor[0.368417, 0.506779, 0.709798]]], 
                    PolygonBox[{
                    Offset[{0, 3.404571913687361}, {0, 0}], 
                    Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                    Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                    Offset[{3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    Offset[{0, -3.404571913687361}, {0, 0}], 
                    
                    Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    
                    Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    
                    Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    Offset[{-3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                    0}]}]}, {DefaultBaseStyle -> {"Graphics", {
                    AbsolutePointSize[6]}, 
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    Dashing[{}]]}, AlignmentPoint -> {0, 0}}], 
                    NCache[
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}], 
                    Scaled[{0.5, 0.5}]], Automatic, 
                    Scaled[1]]}}}, AspectRatio -> Full, ImageSize -> {60, 25},
                     PlotRangePadding -> None, ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.29200000000000004`] -> 
                    Baseline)], #}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{10, 0}, {20, 10}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontSize -> 13, FontFamily -> "Arial"}, Background -> 
               Automatic, StripOnInput -> False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> 
                    RGBColor[0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.368417, 0.506779, 0.709798]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"Dashing", "[", 
                    RowBox[{"{", "}"}], "]"}]}], "}"}], "]"}], "}"}], ",", 
                 RowBox[{"{", #, "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", 
                   RowBox[{"{", 
                    GraphicsBox[{
                    DynamicBox[
                    Typeset`ToBoxes[
                    EdgeForm[
                    Directive[
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    JoinForm["Round"], 
                    AbsoluteThickness[1], 
                    Opacity[1]]], StandardForm, Graphics]], 
                    FaceForm[
                    Directive[
                    Opacity[0.15], 
                    RGBColor[0.368417, 0.506779, 0.709798]]], 
                    PolygonBox[{
                    Offset[{0, 3.404571913687361}, {0, 0}], 
                    Offset[{0.8811680516834245, 3.2885639388890184`}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{3.2885639388890184`, 0.8811680516834245}, {0, 0}], 
                    Offset[{3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    Offset[{0, -3.404571913687361}, {0, 0}], 
                    
                    Offset[{-0.8811680516834245, -3.2885639388890184`}, {0, 
                    0}], 
                    
                    Offset[{-1.7022859568436806`, -2.9484457662642556`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, -2.407395887205594}, {0, 0}], 
                    
                    Offset[{-2.9484457662642556`, -1.7022859568436806`}, {0, 
                    0}], 
                    
                    Offset[{-3.2885639388890184`, -0.8811680516834245}, {0, 
                    0}], 
                    Offset[{-3.404571913687361, 0}, {0, 0}], 
                    
                    Offset[{-3.2885639388890184`, 0.8811680516834245}, {0, 
                    0}], 
                    
                    Offset[{-2.9484457662642556`, 1.7022859568436806`}, {0, 
                    0}], 
                    Offset[{-2.407395887205594, 2.407395887205594}, {0, 0}], 
                    
                    Offset[{-1.7022859568436806`, 2.9484457662642556`}, {0, 
                    0}], 
                    
                    Offset[{-0.8811680516834245, 3.2885639388890184`}, {0, 
                    0}]}]}, AlignmentPoint -> {0, 0}], "}"}]}], ",", 
                 RowBox[{"LegendMarkerSize", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"60", ",", "25"}], "}"}]}], ",", 
                 RowBox[{"LegendMargins", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"10", ",", "0"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"20", ",", "10"}], "}"}]}], "}"}]}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", "13"}]}], "]"}]& ), 
            Editable -> True]}, {""}}, AutoDelete -> False, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
       TraditionalForm], TraditionalForm]},
    "Legended",
    DisplayFunction->(GridBox[{{
        TagBox[
         ItemBox[
          PaneBox[
           TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
           BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
         "SkipImageSizeLevel"], 
        ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
      GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
      AutoDelete -> False, GridBoxItemSize -> Automatic, 
      BaselinePosition -> {1, 1}]& ),
    Editable->True,
    InterpretationFunction->(RowBox[{"Legended", "[", 
       RowBox[{#, ",", 
         RowBox[{"Placed", "[", 
           RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Grenzfrequenz berechen unter voller Korrelation", "Subsubsection"],

Cell["\<\
Als erstes holen wir uns das fittedmodel-object aus dem return.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fitModelObject1", "=", 
  RowBox[{"fit1Result", "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"FittedModel", "[", 
   TagBox[
    PanelBox[
     TagBox[
      RowBox[{
       RowBox[{"-", "1.9074148183059154`*^-19"}], "+", 
       RowBox[{"5.8186117508030976`*^-34", " ", "\[Nu]"}]}],
      Short[#, 2]& ],
     FrameMargins->5],
    Editable -> False], "]"}],
  InterpretTemplate[
  FittedModel[{
    "Linear", {-1.9074148183059154`*^-19, 
     5.8186117508030976`*^-34}, {{$CellContext`\[Nu]}, {
      1, $CellContext`\[Nu]}}, {0, 0}}, {{2.561238409427186*^40, 
    1.202359142203318*^41, 1.202359142203318*^41, 2.2084147509856857`*^40, 
    1.1990285074603723`*^42, 4.809436568813272*^41}}, {{8.213492*^14, 
    2.8118199681*^-19}, {7.402282913580249*^14, 2.4240932260599998`*^-19}, {
    6.87597380733945*^14, 2.1068622553*^-19}, {6.093342642276422*^14, 
    1.61499403296*^-19}, {5.490704358974359*^14, 1.276614330816*^-19}, {
    5.16883548275862*^14, 1.1218440693240002`*^-19}}, {{1., 8.213492*^14}, {
    1., 7.402282913580249*^14}, {1., 6.87597380733945*^14}, {1., 
    6.093342642276422*^14}, {1., 5.490704358974359*^14}, {1., 
    5.16883548275862*^14}}, 
    Function[Null, 
     Internal`LocalizedBlock[{$CellContext`\[Nu]}, #], {HoldAll}]]& ],
  Editable->False,
  SelectWithContents->True,
  Selectable->True]], "Output"]
}, Open  ]],

Cell["\<\
Grenzfrequenz unter voller Korrelation ausrechnen mithilfe des \
fittedmodel-objects mit analyse Darstellung. Danach auch noch Darstellung der \
gerundeten Form.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"grenzfrequenz", "=", 
  RowBox[{"GaussianErrorPropagation", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "Wa"}], "/", "h"}], ",", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"fitModelObject1", ",", 
       RowBox[{"{", 
        RowBox[{"Wa", ",", "h"}], "}"}]}], "}"}], "}"}], ",", 
    RowBox[{"Analyze", "\[Rule]", "True"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"RoundedErrorForm", "@", "grenzfrequenz"}]}], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {"\<\"Parameters\"\>", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"Wa", ",", 
             RowBox[{"-", "1.9074148183059154`*^-19"}], ",", 
             "7.269603308757982`*^-21"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
            "h", ",", "5.8186117508030976`*^-34", ",", 
             "1.2766358257394272`*^-35"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"Wa", ",", "h", ",", 
             RowBox[{"-", "9.217579827703368`*^-56"}]}], "}"}]}], "}"}]}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {
           GrayLevel[0.85], 
           GrayLevel[1]}, "Rows" -> {{None}}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"Function\"\>", "\<\"Uncertanty\"\>"},
        {
         RowBox[{"-", 
          FractionBox["Wa", "h"]}], 
         SqrtBox[
          RowBox[{
           FractionBox[
            RowBox[{
             SuperscriptBox["Wa", "2"], " ", 
             SuperscriptBox[
              RowBox[{"C", "[", "h", "]"}], "2"]}], 
            SuperscriptBox["h", "4"]], "+", 
           FractionBox[
            SuperscriptBox[
             RowBox[{"C", "[", "Wa", "]"}], "2"], 
            SuperscriptBox["h", "2"]], "-", 
           FractionBox[
            RowBox[{"2", " ", "Wa", " ", 
             RowBox[{"C", "[", 
              RowBox[{"Wa", ",", "h"}], "]"}]}], 
            SuperscriptBox["h", "3"]]}]]},
        {"3.278126982854028`*^14", "5.415257613049245`*^12"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"Type/Variable\"\>", "\<\"Formula\"\>", "\<\"Value\"\>", \
"\<\"Weights [%]\"\>", "\<\"Relative Weights [%]\"\>", "\<\"Sqrt-Weights \
[%]\"\>"},
        {
         RowBox[{"C", "[", "Wa", "]"}], 
         FractionBox[
          SuperscriptBox[
           RowBox[{"C", "[", "Wa", "]"}], "2"], 
          SuperscriptBox["h", "2"]], "1.5609272630960918`*^26", 
         "40.40487926027893`", "532.2852391588045`", "37.806534217142655`"},
        {
         RowBox[{"C", "[", "h", "]"}], 
         FractionBox[
          RowBox[{
           SuperscriptBox["Wa", "2"], " ", 
           SuperscriptBox[
            RowBox[{"C", "[", "h", "]"}], "2"]}], 
          SuperscriptBox["h", "4"]], "5.173051966263624`*^25", 
         "13.390536833179974`", "176.40406879574422`", "21.76451302110208`"},
        {
         RowBox[{"C", "[", 
          RowBox[{"Wa", ",", "h"}], "]"}], 
         RowBox[{"-", 
          FractionBox[
           RowBox[{"2", " ", "Wa", " ", 
            RowBox[{"C", "[", 
             RowBox[{"Wa", ",", "h"}], "]"}]}], 
           SuperscriptBox["h", "3"]]}], 
         RowBox[{"-", "1.784982309565576`*^26"}], 
         RowBox[{"-", "46.204583906541096`"}], 
         RowBox[{"-", "608.6893079545487`"}], 
         RowBox[{"-", "40.42895276175527`"}]}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]}
   },
   AutoDelete->False,
   GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
       GrayLevel[0.85], 
       GrayLevel[1], 
       GrayLevel[0.85]}},
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3.278126982854028`*^14", ",", "5.415257613049245`*^12"}], 
  "}"}]], "Output"],

Cell[BoxData["\<\"(3.28\[PlusMinus]0.06)\\!\\(\\*FormBox[TemplateBox[List[\\\"\
10\\\", \\\"14\\\"], \\\"Superscript\\\", Rule[SyntaxForm, SuperscriptBox]], \
TraditionalForm]\\)\"\>"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Beispiel: F1 - Fehlerverteilung: Vorzeichentest", "Section"],

Cell[CellGroupData[{

Cell["Daten eingeben", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dataF1", "=", 
   RowBox[{"{", 
    RowBox[{"0.03", ",", "0.03", ",", "0.06", ",", "0.01", ",", "0.02", ",", 
     RowBox[{"-", "0.04"}], ",", 
     RowBox[{"-", "0.03"}], ",", 
     RowBox[{"-", "0.01"}], ",", "0", ",", 
     RowBox[{"-", "0.02"}], ",", "0", ",", "0.02", ",", 
     RowBox[{"-", "0.02"}], ",", "0.02", ",", 
     RowBox[{"-", "0.02"}], ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.03"}], ",", "0.03", ",", "0", ",", "0.01", ",", 
     RowBox[{"-", "0.07"}], ",", "0.04", ",", "0.03", ",", "0.03", ",", 
     RowBox[{"-", "0.03"}], ",", 
     RowBox[{"-", "0.01"}], ",", "0", ",", "0.01", ",", 
     RowBox[{"-", "0.03"}], ",", "0", ",", 
     RowBox[{"-", "0.04"}], ",", 
     RowBox[{"-", "0.04"}], ",", "0", ",", 
     RowBox[{"-", "0.06"}], ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.06"}], ",", 
     RowBox[{"-", "0.01"}], ",", "0.03", ",", "0.01", ",", 
     RowBox[{"-", "0.03"}], ",", "0", ",", "0", ",", "0.01", ",", "0.02", ",", 
     RowBox[{"-", "0.01"}], ",", "0.06", ",", 
     RowBox[{"-", "0.03"}], ",", 
     RowBox[{"-", "0.04"}], ",", "0.02", ",", "0.01", ",", 
     RowBox[{"-", "0.02"}], ",", "0.02", ",", 
     RowBox[{"-", "0.02"}], ",", "0", ",", "0", ",", "0", ",", 
     RowBox[{"-", "0.02"}], ",", "0.04", ",", "0", ",", 
     RowBox[{"-", "0.08"}], ",", 
     RowBox[{"-", "0.06"}], ",", 
     RowBox[{"-", "0.03"}], ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.03"}], ",", "0.03", ",", 
     RowBox[{"-", "0.02"}], ",", 
     RowBox[{"-", "0.04"}], ",", 
     RowBox[{"-", "0.02"}], ",", "0", ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.01"}], ",", "0.07", ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.03"}], ",", "0.01", ",", "0.03", ",", "0.02", ",", 
     RowBox[{"-", "0.02"}], ",", 
     RowBox[{"-", "0.03"}], ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.04"}], ",", "0.03", ",", "0.02", ",", "0.01", ",", "0", 
     ",", 
     RowBox[{"-", "0.05"}], ",", 
     RowBox[{"-", "0.01"}], ",", 
     RowBox[{"-", "0.01"}], ",", "0.04", ",", "0.03", ",", "0.02", ",", 
     RowBox[{"-", "0.05"}], ",", "0.01", ",", 
     RowBox[{"-", "0.03"}], ",", "0", ",", "0", ",", "0.02", ",", "0.06", ",",
      "0", ",", "0.01"}], "}"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Daten analysieren mit Zuf\[ADoubleDot]lligen Fehler", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalculateMeasurementHU", "[", 
  RowBox[{"dataF1", ",", 
   RowBox[{"Analyze", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[GridBox[{
        {"\<\"\\!\\(\\*OverscriptBox[\\(x\\), \\(_\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(u\\), \\(sys\\)]\\)\"\>", \
"\<\"\\!\\(\\*OverscriptBox[\\(x\\), \
\\(_\\)]\\)+\\!\\(\\*SubscriptBox[\\(u\\), \\(sys\\)]\\)\"\>", "\<\"u\"\>", "\
\<\"\\!\\(\\*SubscriptBox[\\(u\\), \
\\(sys\\)]\\)/\\!\\(\\*OverscriptBox[\\(x\\), \\(_\\)]\\) [%]\"\>", \
"\<\"n\"\>", "\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(+\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(-\\)]\\)\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(n\\), \\(0\\)]\\)\"\>", \
"\<\"|\\!\\(\\*SubscriptBox[\\(n\\), \
\\(+\\)]\\)-\\!\\(\\*SubscriptBox[\\(n\\), \
\\(-\\)]\\)|\[LessEqual]\\!\\(\\*SqrtBox[\\(n\\)]\\)\"\>", \
"\<\"\[Sigma]\"\>", "\<\"\\!\\(\\*SubscriptBox[\\(min\\), \
\\(i\\)]\\)|\\!\\(\\*OverscriptBox[\\(x\\), \
\\(_\\)]\\)-\\!\\(\\*SubscriptBox[\\(x\\), \\(i\\)]\\)|\"\>", \
"\<\"\\!\\(\\*SubscriptBox[\\(max\\), \\(i\\)]\\)|\\!\\(\\*OverscriptBox[\\(x\
\\), \\(_\\)]\\)-\\!\\(\\*SubscriptBox[\\(x\\), \\(i\\)]\\)|\"\>"},
        {
         RowBox[{"-", "0.0034`"}], "0", 
         RowBox[{"-", "0.0034`"}], "0.0029274734858066626`", "0.`", "100", 
         "46", "54", "0", 
         ItemBox["\<\"8\[LessEqual]10.\"\>",
          Background->RGBColor[0.88, 1, 0.88],
          StripOnInput->False], "0.029274734858066625`", "0.0034`", "0.0766`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     TagBox[GridBox[{
        {"\<\"uncertanity u\"\>", "\<\"\\!\\(\\*OverscriptBox[\\(s\\), \
\\(_\\)]\\)=s/\\!\\(\\*SqrtBox[\\(n\\)]\\)\"\>", "0"},
        {"\<\"N[u]\"\>", "0.0029274734858066626`", "0"},
        {"\<\"u^2\"\>", "8.570101010101012`*^-6", "0"},
        {"\<\"uncertanty weights [%]\"\>", "100.`", "0.`"},
        {"\<\"Sqrt-uncertanty weights [%]\"\>", "100.`", "0.`"}
       },
       AutoDelete->False,
       GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
           GrayLevel[0.85], 
           GrayLevel[1], 
           GrayLevel[0.85], 
           GrayLevel[1]}},
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
      "Grid"]},
    {
     GraphicsBox[{{
        {RGBColor[0.92, 0.488, 0.2432], EdgeForm[{Opacity[0.63], Thickness[
         Small]}], {}, 
         {RGBColor[0.92, 0.488, 0.2432], EdgeForm[{Opacity[0.63], Thickness[
          Small]}], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{-0.08, 0}, {-0.06, 1}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{183.30739690931858`, 228.7663011105409}, {
               94.32084116257732, 110.75000000000001`}}],
             StatusArea[#, 1]& ,
             TagBoxNote->"1"],
            StyleBox["1", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[1, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{-0.06, 0}, 
                NCache[{-0.04, 
                  Rational[5, 2]}, {-0.04, 2.5}], "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{228.2663011105409, 273.72520531176315`}, {
               70.42710290644328, 110.75000000000001`}}],
             StatusArea[#, 2.5]& ,
             TagBoxNote->"2.5"],
            StyleBox["2.5`", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[2.5, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{-0.04, 0}, 
                NCache[{-0.02, 
                  Rational[17, 2]}, {-0.02, 8.5}], "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{273.22520531176315`, 
               318.68410951298546`}, {-25.147850118092848`, 
               110.75000000000001`}}],
             StatusArea[#, 8.5]& ,
             TagBoxNote->"8.5"],
            StyleBox["8.5`", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[8.5, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{-0.02, 0}, {0., 11}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{318.18410951298546`, 
               363.64301371420777`}, {-64.9707472116496, 
               110.75000000000001`}}],
             StatusArea[#, 11]& ,
             TagBoxNote->"11"],
            StyleBox["11", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[11, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{0., 0}, 
                NCache[{0.02, 
                  Rational[27, 2]}, {0.02, 13.5}], "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{363.14301371420777`, 
               408.6019179154301}, {-104.79364430520631`, 
               110.75000000000001`}}],
             StatusArea[#, 13.5]& ,
             TagBoxNote->"13.5"],
            StyleBox["13.5`", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[13.5, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{0.02, 0}, {0.04, 10}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{408.1019179154301, 
               453.5608221166524}, {-49.04158837422689, 110.75000000000001`}}],
             StatusArea[#, 10]& ,
             TagBoxNote->"10"],
            StyleBox["10", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[10, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{0.04, 0}, 
                NCache[{0.06, 
                  Rational[3, 2]}, {0.06, 1.5}], "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{453.0608221166524, 498.51972631787464`}, {
               86.35626174386599, 110.75000000000001`}}],
             StatusArea[#, 1.5]& ,
             TagBoxNote->"1.5"],
            StyleBox["1.5`", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[1.5, {FontFamily -> "Times"}], "Tooltip"]& ], 
          TagBox[
           TooltipBox[
            TagBox[
             DynamicBox[{
               FEPrivate`If[
                CurrentValue["MouseOver"], 
                EdgeForm[{
                  GrayLevel[0.5], 
                  AbsoluteThickness[1.5], 
                  Opacity[0.66]}], {}, {}], 
               RectangleBox[{0.06, 0}, {0.08, 2}, "RoundingRadius" -> 0]},
              
              ImageSizeCache->{{498.01972631787464`, 543.4786305190969}, {
               78.39168232515463, 110.75000000000001`}}],
             StatusArea[#, 2]& ,
             TagBoxNote->"2"],
            StyleBox["2", {FontFamily -> "Times"}, StripOnInput -> False]],
           Annotation[#, 
            Style[2, {FontFamily -> "Times"}], 
            "Tooltip"]& ]}, {}, {}}, {{}, {}, {}, {}, {}, {}, {}, {}}}, {{}, \
{{}, {}, 
         {RGBColor[1, 0, 0], PointSize[0.008333333333333333], 
          AbsoluteThickness[1.6], CapForm["Butt"], LineBox[CompressedData["
1:eJw9e3k4lG/cPVkTIhLZd5KlohT53JWkqGxFkqjEN3sJyb5EUZF93/d9zDC2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           "]]}}, {}, {}, {{}, {}}}, {{{}, {}, 
         {RGBColor[0, 0, 1], AbsoluteThickness[1.6], Opacity[1.], CapForm[
          "Butt"], LineBox[CompressedData["
1:eJw1mnk4Vd/3x4U+5iFTZCjKeBUhZFpHNBvKHDJFhDIPiTKTZMwQMiVknqfL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           "]]}}, {}, {}}},
      AspectRatio->NCache[
        Rational[1, 3], 0.3333333333333333],
      Axes->{False, False},
      AxesLabel->{None, None},
      AxesOrigin->{-0.15562862126194643`, 0},
      Epilog->{
        GrayLevel[0.5], 
        Thickness[Large], 
        TagBox[
         ConicHullRegionBox[{{-0.0034, 0}, {-0.0034, 1}}], "InfiniteLine"], 
        Thickness[Tiny], 
        TagBox[
         ConicHullRegionBox[{{-0.03267473485806662, 0}, {-0.03267473485806662,
             1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{0.025874734858066625`, 0}, {
           0.025874734858066625`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{-0.06194946971613325, 0}, {-0.06194946971613325,
             1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{0.05514946971613325, 0}, {
           0.05514946971613325, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{-0.09122420457419987, 0}, {-0.09122420457419987,
             1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{0.08442420457419987, 0}, {
           0.08442420457419987, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{-0.1204989394322665, 0}, {-0.1204989394322665, 
            1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{0.1136989394322665, 0}, {
           0.1136989394322665, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{-0.14977367429033311`, 
            0}, {-0.14977367429033311`, 1}}], "InfiniteLine"], 
        TagBox[
         ConicHullRegionBox[{{0.14297367429033314`, 0}, {
           0.14297367429033314`, 1}}], "InfiniteLine"]},
      Frame->True,
      FrameLabel->{{None, None}, {None, None}},
      FrameStyle->Automatic,
      FrameTicks->{{Automatic, Automatic}, {Automatic, {{-0.0034, 
           FormBox["\"\[Mu]\"", TraditionalForm]}, {0.025874734858066625`, 
           FormBox["\"1\[Sigma]\"", TraditionalForm]}, {-0.03267473485806662, 
           FormBox["\"-1\[Sigma]\"", TraditionalForm]}, {0.05514946971613325, 
           FormBox["\"2\[Sigma]\"", TraditionalForm]}, {-0.06194946971613325, 
           FormBox["\"-2\[Sigma]\"", TraditionalForm]}, {0.08442420457419987, 
           FormBox["\"3\[Sigma]\"", TraditionalForm]}, {-0.09122420457419987, 
           FormBox["\"-3\[Sigma]\"", TraditionalForm]}, {0.1136989394322665, 
           FormBox["\"4\[Sigma]\"", TraditionalForm]}, {-0.1204989394322665, 
           FormBox["\"-4\[Sigma]\"", TraditionalForm]}, {0.14297367429033314`, 
           FormBox["\"5\[Sigma]\"", TraditionalForm]}, {-0.14977367429033311`, 
           FormBox["\"-5\[Sigma]\"", TraditionalForm]}}}},
      GridLines->{{0}, {0}},
      GridLinesStyle->Directive[
        GrayLevel[0.5, 0.4]],
      ImageSize->700,
      LabelStyle->{FontFamily -> "Times"},
      PlotRange->{{-0.14977367429033311`, 0.14297367429033314`}, {All, All}},
      PlotRangePadding->{{
         Scaled[0.02], 
         Scaled[0.02]}, {None, 
         Scaled[0.05]}},
      Ticks->{Automatic, Automatic}]}
   },
   AutoDelete->False,
   GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
       GrayLevel[1], 
       GrayLevel[0.85]}},
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.0034`"}], ",", "0.0029274734858066626`"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell["\<\
Vorzeichentest ist gr\[UDoubleDot]n mit 8<=10 und Vorzeichentest ist demnach \
bestanden.\
\>", "Subsection"]
}, Open  ]]
},
WindowSize->{1920, 1007},
WindowMargins->{{-8, Automatic}, {Automatic, -8}},
FrontEndVersion->"11.0 for Microsoft Windows (64-bit) (July 28, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 84, 1, 63, "Section"],
Cell[667, 25, 106, 3, 30, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[810, 33, 37, 0, 63, "Section"],
Cell[850, 35, 72, 0, 30, "Text"],
Cell[925, 37, 186, 4, 69, "Input"],
Cell[1114, 43, 59, 0, 30, "Text"],
Cell[1176, 45, 52, 1, 30, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1265, 51, 58, 0, 63, "Section"],
Cell[CellGroupData[{
Cell[1348, 55, 38, 0, 34, "Subsubsection"],
Cell[1389, 57, 257, 7, 30, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1683, 69, 184, 3, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[1892, 76, 376, 10, 88, "Input"],
Cell[2271, 88, 98, 2, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2418, 96, 55, 0, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[2498, 100, 351, 10, 30, "Input"],
Cell[2852, 112, 37938, 678, 438, "Print"],
Cell[40793, 792, 98, 2, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[40940, 800, 69, 0, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[41034, 804, 356, 11, 30, "Input"],
Cell[41393, 817, 196, 2, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[41650, 826, 143, 3, 63, "Section"],
Cell[41796, 831, 28, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[41849, 835, 956, 26, 145, "Input"],
Cell[42808, 863, 114, 3, 30, "Output"]
}, Open  ]],
Cell[42937, 869, 27, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[42989, 873, 951, 26, 145, "Input"],
Cell[43943, 901, 6936, 183, 327, "Print"],
Cell[50882, 1086, 114, 3, 30, "Output"]
}, Open  ]],
Cell[51011, 1092, 48, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[51084, 1096, 71, 1, 30, "Input"],
Cell[51158, 1099, 59, 0, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[51266, 1105, 149, 3, 63, "Section"],
Cell[CellGroupData[{
Cell[51440, 1112, 39, 0, 34, "Subsubsection"],
Cell[51482, 1114, 1327, 37, 69, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52846, 1156, 65, 0, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[52936, 1160, 683, 16, 30, "Input"],
Cell[53622, 1178, 54680, 1109, 484, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[108351, 2293, 90, 1, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[108466, 2298, 262, 6, 30, "Input"],
Cell[108731, 2306, 41208, 815, 362, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[149988, 3127, 72, 0, 34, "Subsubsection"],
Cell[150063, 3129, 87, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[150175, 3135, 149, 4, 30, "Input"],
Cell[150327, 3141, 1275, 30, 51, "Output"]
}, Open  ]],
Cell[151617, 3174, 185, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[151827, 3182, 462, 12, 50, "Input"],
Cell[152292, 3196, 4072, 111, 253, "Print"],
Cell[156367, 3309, 118, 3, 34, "Output"],
Cell[156488, 3314, 195, 2, 30, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[156744, 3323, 66, 0, 63, "Section"],
Cell[CellGroupData[{
Cell[156835, 3327, 39, 0, 34, "Subsubsection"],
Cell[156877, 3329, 2328, 52, 69, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[159242, 3386, 76, 0, 34, "Subsubsection"],
Cell[CellGroupData[{
Cell[159343, 3390, 142, 3, 30, "Input"],
Cell[159488, 3395, 45079, 833, 431, "Print"],
Cell[204570, 4230, 119, 3, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[204716, 4237, 119, 3, 43, "Subsection"]
}, Open  ]]
}
]
*)

